@@ -67,9 +67,6 @@ const Application = new Lang.Class({
           { name: 'message-user',
             activate: Lang.bind(this, this._onMessageUser),
             parameter_type: GLib.VariantType.new('(sssu)') },
-          { name: 'paste-text',
-            activate: Lang.bind(this, this._onPasteText),
-            parameter_type: GLib.VariantType.new('s') },
           { name: 'leave-room',
             activate: Lang.bind(this, this._onLeaveRoom),
             parameter_type: GLib.VariantType.new('(ss)') },
@@ -77,6 +74,8 @@ const Application = new Lang.Class({
             activate: Lang.bind(this, this._onLeaveCurrentRoom),
             create_hook: Lang.bind(this, this._leaveRoomCreateHook),
             accels: ['<Primary>w'] },
+          { name: 'authenticate-account',
+            parameter_type: GLib.VariantType.new('(os)') },
           { name: 'reconnect-account',
             parameter_type: GLib.VariantType.new('o') },
           { name: 'user-list',
@@ -106,7 +105,11 @@ const Application = new Lang.Class({
           { name: 'last-room',
             accels: ['<Primary>End'] },
           { name: 'nth-room',
-            parameter_type: GLib.VariantType.new('i') }
+            parameter_type: GLib.VariantType.new('i') },
+          { name: 'next-pending-room',
+            accels: ['<Alt><Shift>Down', '<Primary><Shift>Page_Down']},
+          { name: 'previous-pending-room',
+            accels: ['<Alt><Shift>Up', '<Primary><Shift>Page_Up']}
         ];
         actionEntries.forEach(Lang.bind(this,
             function(actionEntry) {
@@ -135,18 +138,18 @@ const Application = new Lang.Class({
 
     vfunc_activate: function() {
         if (!this._window) {
-            this._window = new MainWindow.MainWindow(this);
-            this._window.window.connect('destroy', Lang.bind(this,
+            this._window = new MainWindow.MainWindow({ application: this });
+            this._window.connect('destroy', Lang.bind(this,
                 function() {
                     for (let id in this._pendingRequests)
                         this._pendingRequests[id].cancellable.cancel();
                     this.emitJS('prepare-shutdown');
             }));
-            this._window.window.show_all();
+            this._window.show_all();
 
             this._chatroomManager.lateInit();
         }
-        this._window.window.present();
+        this._window.present();
     },
 
     _updateAccountAction: function(action) {
@@ -326,11 +329,13 @@ const Application = new Lang.Class({
             }
 
             if (error && error != ConnectionError.CANCELLED)
-                logError(e);
+                Utils.debug('Account %s disconnected with error %s'.format(
+                            account.get_path_suffix(),
+                            error.replace(Tp.ERROR_PREFIX + '.', '')));
         } catch (e if e.matches(Tp.Error, Tp.Error.CANCELLED)) {
             // interrupted by user request, don't log
         } catch (e) {
-            logError(e, 'Failed to ensure channel');
+            Utils.debug('Failed to ensure channel: ' + e.message);
         }
 
         if (requestData.callback)
@@ -374,11 +379,6 @@ const Application = new Lang.Class({
         }));
     },
 
-    _onPasteText: function(action, parameter) {
-        let text = parameter.deep_unpack();
-        this.pasteManager.pasteText(text);
-    },
-
     _onLeaveRoom: function(action, parameter) {
         let [roomId, message] = parameter.deep_unpack();
         let reason = Tp.ChannelGroupChangeReason.NONE;
@@ -442,13 +442,13 @@ const Application = new Lang.Class({
         let accountPath = parameter.deep_unpack();
         let factory = Tp.AccountManager.dup().get_factory();
         let account = factory.ensure_account(accountPath, []);
-        let dialog = new Connections.ConnectionDetailsDialog(account);
-        dialog.widget.transient_for = this._window.window;
-        dialog.widget.connect('response', Lang.bind(this,
+        let dialog = new Connections.ConnectionProperties(account);
+        dialog.transient_for = this._window;
+        dialog.connect('response', Lang.bind(this,
             function(w, response) {
                 w.destroy();
             }));
-        dialog.widget.show();
+        dialog.show();
     },
 
     _onShowPreferences: function() {
@@ -471,7 +471,8 @@ const Application = new Lang.Class({
                 'Kunaal Jain <kunaalus@gmail.com>',
                 'Cody Welsh <codyw@protonmail.com>',
                 'Isabella Ribeiro <belinhacbr@gmail.com>',
-                'Jonas Danielsson <jonas@threetimestwo.org>'
+                'Jonas Danielsson <jonas@threetimestwo.org>',
+                'Rares Visalom <rares.visalom@gmail.com>'
             ],
             artists: [
                 'Sam Hewitt',
@@ -486,7 +487,7 @@ const Application = new Lang.Class({
             website_label: _("Learn more about Polari"),
             website: 'https://wiki.gnome.org/Apps/Polari',
 
-            transient_for: this._window.window,
+            transient_for: this._window,
             modal: true
         };
 
@@ -499,7 +500,7 @@ const Application = new Lang.Class({
     },
 
     _onQuit: function() {
-        this._window.window.destroy();
+        this._window.destroy();
     }
 });
 Utils.addJSSignalMethods(Application.prototype);
