@@ -1,4 +1,6 @@
 const Gdk = imports.gi.Gdk;
+const GdkPixbuf = imports.gi.GdkPixbuf;
+const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
@@ -9,6 +11,8 @@ const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const TabCompletion = imports.tabCompletion;
 const Tp = imports.gi.TelepathyGLib;
+const PasteManager = imports.pasteManager;
+const Utils = imports.utils;
 
 const MAX_NICK_UPDATE_TIME = 5; /* s */
 const MAX_LINES = 5;
@@ -18,7 +22,8 @@ const ChatEntry = new Lang.Class({
     Name: 'ChatEntry',
     Extends: Gtk.Entry,
     Signals: { 'text-pasted': { param_types: [GObject.TYPE_STRING,
-                                              GObject.TYPE_INT] } },
+                                              GObject.TYPE_INT] },
+               'image-pasted': { param_types: [GdkPixbuf.Pixbuf.$gtype] } },
 
     _init: function(params) {
         this.parent(params);
@@ -35,6 +40,10 @@ const ChatEntry = new Lang.Class({
         let clipboard = Gtk.Clipboard.get_default(this.get_display());
         clipboard.request_text(Lang.bind(this,
             function(clipboard, text) {
+                if (text == null)
+                    return;
+            	text = text.trim();
+
                 let nLines = text.split('\n').length;
                 if (nLines >= MAX_LINES) {
                     this.emit('text-pasted', text, nLines);
@@ -45,23 +54,38 @@ const ChatEntry = new Lang.Class({
                 this.emit('paste-clipboard');
                 this._useDefaultHandler = false;
             }));
+
+        clipboard.request_image(Lang.bind(this,
+            function(clipboard, pixbuf) {
+                if (pixbuf == null)
+                    return;
+                this.emit('image-pasted', pixbuf);
+            }));
     },
 });
 
 const EntryArea = new Lang.Class({
     Name: 'EntryArea',
     Extends: Gtk.Stack,
-    Template: 'resource:///org/gnome/Polari/entry-area.ui',
+    Template: 'resource:///org/gnome/Polari/ui/entry-area.ui',
     InternalChildren: ['chatEntry',
                        'nickButton',
                        'nickLabel',
                        'nickPopover',
                        'nickEntry',
                        'changeButton',
-                       'multiLineBox',
-                       'multiLineLabel',
+                       'pasteBox',
+                       'confirmLabel',
+                       'uploadLabel',
                        'cancelButton',
                        'pasteButton'],
+    Properties: {
+        'max-nick-chars': GObject.ParamSpec.uint('max-nick-chars',
+                                                 'max-nick-chars',
+                                                 'max-nick-chars',
+                                                 GObject.ParamFlags.WRITABLE,
+                                                 0, GLib.MAXUINT32, 0)
+    },
 
     _init: function(params) {
         this._room = params.room;
@@ -93,6 +117,7 @@ const EntryArea = new Lang.Class({
         this._nickPopover.set_default_widget(this._changeButton);
 
         this._chatEntry.connect('text-pasted', Lang.bind(this, this._onTextPasted));
+        this._chatEntry.connect('image-pasted', Lang.bind(this, this._onImagePasted));
         this._chatEntry.connect('changed', Lang.bind(this, this._onEntryChanged));
 
         this._chatEntry.connect('activate', Lang.bind(this,
@@ -105,10 +130,10 @@ const EntryArea = new Lang.Class({
                 }
             }));
 
-        this._cancelButton.connect('clicked', Lang.bind(this, this._onButtonClicked));
-        this._pasteButton.connect('clicked', Lang.bind(this, this._onButtonClicked));
+        this._cancelButton.connect('clicked', Lang.bind(this, this._onCancelClicked));
+        this._pasteButton.connect('clicked', Lang.bind(this, this._onPasteClicked));
 
-        this._multiLineBox.connect_after('key-press-event', Lang.bind(this,
+        this._pasteBox.connect_after('key-press-event', Lang.bind(this,
             function(w, event) {
                 let [, keyval] = event.get_keyval();
                 let [, mods] = event.get_state();
@@ -137,7 +162,7 @@ const EntryArea = new Lang.Class({
         this._chatEntry.connect('unmap', Lang.bind(this, this._updateCompletions));
     },
 
-    set maxNickChars(maxChars) {
+    set max_nick_chars(maxChars) {
         this._maxNickChars = maxChars;
         this._updateNick();
     },
@@ -155,17 +180,16 @@ const EntryArea = new Lang.Class({
         this._completion.setCompletions(nicks);
     },
 
+    _canFocusChatEntry: function() {
+        let toplevelFocus = this._chatEntry.get_toplevel().get_focus();
+        return this.sensitive &&
+               this._chatEntry.get_mapped() &&
+               !this._chatEntry.has_focus &&
+               !(toplevelFocus instanceof Gtk.Entry);
+    },
+
     _onKeyPressEvent: function(w, event) {
-        if (!this._chatEntry.get_mapped())
-            return Gdk.EVENT_PROPAGATE;
-
-        if (!this.sensitive)
-            return Gdk.EVENT_PROPAGATE;
-
-        if (this._chatEntry.has_focus)
-            return Gdk.EVENT_PROPAGATE;
-
-        if (this._chatEntry.get_toplevel().get_focus() instanceof Gtk.Entry)
+        if (!this._canFocusChatEntry())
             return Gdk.EVENT_PROPAGATE;
 
         let [, keyval] = event.get_keyval();
@@ -194,30 +218,70 @@ const EntryArea = new Lang.Class({
         this._chatEntry.get_style_context().remove_class('error');
     },
 
+    _setPasteContent: function(content) {
+        this._pasteContent = content;
+
+        if (content) {
+            this._confirmLabel.show();
+            this.visible_child_name = 'paste-confirmation';
+            this._pasteButton.grab_focus();
+        } else {
+            this.visible_child_name = 'default';
+            this._chatEntry.grab_focus_without_selecting();
+        }
+    },
+
     _onTextPasted: function(entry, text, nLines) {
-        this._multiLineLabel.label =
+        this._confirmLabel.label =
             ngettext("Paste %s line of text to public paste service?",
                      "Paste %s lines of text to public paste service?",
                      nLines).format(nLines);
-        this._pasteButton.action_target = new GLib.Variant('s', text);
-        this.visible_child_name = 'multiline';
-        this._pasteButton.grab_focus();
+        this._uploadLabel.label =
+            ngettext("Uploading %s line of text to public paste service …",
+                     "Uploading %s lines of text to public paste service …",
+                     nLines).format(nLines);
+        this._setPasteContent(text);
     },
 
-    _onButtonClicked: function() {
-            this._chatEntry.text = '';
-            this.visible_child_name = 'default';
+    _onImagePasted: function(entry, pixbuf) {
+        this._confirmLabel.label = _("Upload image to public paste service?");
+        this._uploadLabel.label = _("Uploading image to public paste service …");
+        this._setPasteContent(pixbuf);
+    },
+
+    _onPasteClicked: function() {
+        let title;
+        let nick = this._room.channel.connection.self_contact.alias;
+        if (this._room.type == Tp.HandleType.ROOM)
+            /* translators: %s is a nick, #%s a channel */
+            title = _("%s in #%s").format(nick, this._room.display_name);
+        else
+            title = _("Paste from %s").format(nick);
+
+        let app = Gio.Application.get_default();
+        try {
+            app.pasteManager.pasteContent(this._pasteContent, title,
+                Lang.bind(this, function(url) {
+                    // TODO: handle errors
+                    this._setPasteContent(null);
+                    if (url)
+                        this._chatEntry.emit('insert-at-cursor', url);
+                }));
+        } catch(e) {
+            let type = typeof this._pasteContent;
+            Utils.debug('Failed to paste content of type ' +
+                        (type == 'object' ? this._pasteContent.toString() : type));
+        }
+        this._confirmLabel.hide();
+    },
+
+    _onCancelClicked: function() {
+        this._setPasteContent(null);
     },
 
     _onSensitiveChanged: function() {
-        if (!this.sensitive)
-            return;
-
-        Mainloop.idle_add(Lang.bind(this,
-            function() {
-                this._chatEntry.grab_focus();
-                return GLib.SOURCE_REMOVE;
-            }));
+        if (this._canFocusChatEntry())
+            this._chatEntry.grab_focus();
     },
 
     _onChannelChanged: function(room) {
