@@ -1,3 +1,4 @@
+const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
@@ -17,11 +18,46 @@ const DialogPage = {
 
 const JoinDialog = new Lang.Class({
     Name: 'JoinDialog',
+    Extends: Gtk.Dialog,
+    Template: 'resource:///org/gnome/Polari/ui/join-room-dialog.ui',
+    InternalChildren: ['cancelButton',
+                       'joinButton',
+                       'mainStack',
+                       'connectionCombo',
+                       'connectionButton',
+                       'nameEntry',
+                       'nameCompletion',
+                       'connectionStack',
+                       'filterEntry',
+                       'connectionsList',
+                       'details',
+                       'addButton',
+                       'customToggle'],
 
-    _init: function() {
-        this._createWidget();
+    _init: function(params) {
+        params['use-header-bar'] = 1;
+        this.parent(params);
 
-        this._settings = new Gio.Settings({ schema_id: 'org.gnome.Polari' });
+        // TODO: Is there really no way to do this in the template?
+        let icon = new Gtk.Image({ icon_name: 'go-previous-symbolic' });
+        this._backButton = new Gtk.Button({ image: icon,
+                                            valign: Gtk.Align.CENTER,
+                                            focus_on_click: false });
+        this.get_header_bar().pack_start(this._backButton);
+
+        let accelGroup = new Gtk.AccelGroup();
+        this._connectionButton.add_accelerator('clicked', accelGroup,
+                                               Gdk.KEY_n,
+                                               Gdk.ModifierType.CONTROL_MASK, 0);
+        this._backButton.add_accelerator('clicked', accelGroup,
+                                         Gdk.KEY_Left,
+                                         Gdk.ModifierType.MOD1_MASK, 0);
+        this.add_accel_group(accelGroup);
+
+        this._setupMainPage();
+        this._setupConnectionPage();
+
+        this._setPage(DialogPage.MAIN);
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._roomManager = ChatroomManager.getDefault();
@@ -46,71 +82,83 @@ const JoinDialog = new Lang.Class({
                     this._updateConnectionCombo();
                 }));
 
-        this.widget.connect('response', Lang.bind(this,
+        this.connect('response', Lang.bind(this,
             function(w, response) {
                 if (response == Gtk.ResponseType.OK)
-                    this._onConfirmClicked();
-                else
-                    this.widget.destroy();
+                    this._joinRoom();
+                this.destroy();
             }));
-        this.widget.connect('destroy', Lang.bind(this,
+        this.connect('destroy', Lang.bind(this,
             function() {
                 this._accountsMonitor.disconnect(this._accountAddedId);
                 this._accountsMonitor.disconnect(this._accountRemovedId);
             }));
 
         this._updateConnectionCombo();
-        this._updateCanConfirm();
-
-        this._nameEntry.grab_focus();
+        this._updateCanJoin();
     },
 
-    _createWidget: function() {
-        let builder = new Gtk.Builder();
-        builder.add_from_resource('/org/gnome/Polari/join-room-dialog.ui');
-
-        this.widget = builder.get_object('join_room_dialog');
-
-        this._stack = builder.get_object('stack');
-
-        this._details = new Connections.ConnectionDetails();
-        this._stack.add_named(this._details, 'connection');
-        this._details.connect('notify::can-confirm',
-                              Lang.bind(this, this._updateCanConfirm));
-        this._details.connect('account-created', Lang.bind(this,
-            function(details, account) {
-                this._connectionCombo.set_active_id(account.display_name);
-            }));
-
-        this._connectionButton = builder.get_object('add_connection_button');
+    _setupMainPage: function() {
         this._connectionButton.connect('clicked', Lang.bind(this,
             function() {
                 this._setPage(DialogPage.CONNECTION);
             }));
 
-        let icon = new Gtk.Image({ icon_name: 'go-previous-symbolic' });
-        this._backButton = new Gtk.Button({ image: icon,
-                                            valign: Gtk.Align.CENTER });
-        this._backButton.connect('clicked', Lang.bind(this,
-            function() {
-                this._setPage(DialogPage.MAIN);
-            }));
-        this.widget.get_header_bar().pack_start(this._backButton);
-
-        this._connectionCombo = builder.get_object('connection_combo');
         this._connectionCombo.connect('changed',
                                       Lang.bind(this, this._onAccountChanged));
         this._connectionCombo.sensitive = false;
 
-        this._confirmButton = builder.get_object('confirm_button');
-        this._cancelButton = builder.get_object('cancel_button');
-
-        this._nameCompletion = builder.get_object('name_completion');
-        this._nameEntry = builder.get_object('name_entry');
         this._nameEntry.connect('changed',
-                                Lang.bind(this, this._updateCanConfirm));
+                                Lang.bind(this, this._updateCanJoin));
+    },
 
-        this._setPage(DialogPage.MAIN);
+    _setupConnectionPage: function() {
+        this._backButton.connect('clicked', Lang.bind(this,
+            function() {
+                this._setPage(DialogPage.MAIN);
+            }));
+        this._connectionsList.connect('account-selected', Lang.bind(this,
+            function() {
+                this._setPage(DialogPage.MAIN);
+            }));
+        this._addButton.connect('clicked', Lang.bind(this,
+            function() {
+                this._details.save();
+                this._setPage(DialogPage.MAIN);
+            }));
+
+        this._connectionsList.connect('account-created',
+                                      Lang.bind(this, this._onAccountCreated));
+        this._details.connect('account-created',
+                              Lang.bind(this, this._onAccountCreated));
+
+        this._customToggle.connect('notify::active', Lang.bind(this,
+            function() {
+                let isCustom = this._customToggle.active;
+                this._connectionStack.visible_child_name = isCustom ? 'custom'
+                                                                    : 'predefined';
+                if (isCustom) {
+                    this._addButton.grab_default();
+                    this._details.reset();
+                }
+            }));
+
+        this._filterEntry.connect('search-changed', Lang.bind(this,
+            function() {
+                this._connectionsList.setFilter(this._filterEntry.text);
+            }));
+        this._filterEntry.connect('stop-search', Lang.bind(this,
+            function() {
+                if (this._filterEntry.text.length > 0)
+                    this._filterEntry.text = '';
+                else
+                    this.response(Gtk.ResponseType.CANCEL);
+            }));
+        this._filterEntry.connect('activate', Lang.bind(this,
+            function() {
+                if (this._filterEntry.text.length > 0)
+                    this._connectionsList.activateFirst();
+            }));
     },
 
     _onAccountChanged: function() {
@@ -143,18 +191,12 @@ const JoinDialog = new Lang.Class({
             }));
     },
 
-    _onConfirmClicked: function() {
-        if (this._page == DialogPage.MAIN) {
-            this._joinRoom();
-            this.widget.destroy();
-        } else {
-            this._details.save();
-            this._setPage(DialogPage.MAIN);
-        }
+    _onAccountCreated: function(w, account) {
+        this._connectionCombo.set_active_id(account.display_name);
     },
 
     _joinRoom: function() {
-        this.widget.hide();
+        this.hide();
 
         let selected = this._connectionCombo.get_active_text();
         let account = this._accounts[selected];
@@ -190,23 +232,19 @@ const JoinDialog = new Lang.Class({
         this._connectionCombo.set_active(activeIndex);
     },
 
-    _updateCanConfirm: function() {
-        let sensitive;
-
-        if (this._page == DialogPage.MAIN) {
+    _updateCanJoin: function() {
+        let sensitive = false;
+        if (this._page == DialogPage.MAIN)
             sensitive = this._connectionCombo.get_active() > -1  &&
                         this._nameEntry.get_text_length() > 0;
-        } else {
-            sensitive = this._details.can_confirm;
-        }
 
-        this._confirmButton.sensitive = sensitive;
-        this.widget.set_default_response(sensitive ? Gtk.ResponseType.OK
-                                                   : Gtk.ResponseType.NONE);
+        this._joinButton.sensitive = sensitive;
+        this.set_default_response(sensitive ? Gtk.ResponseType.OK
+                                            : Gtk.ResponseType.NONE);
     },
 
     get _page() {
-        if (this._stack.visible_child_name == 'connection')
+        if (this._mainStack.visible_child_name == 'connection')
             return DialogPage.CONNECTION;
         else
             return DialogPage.MAIN;
@@ -216,15 +254,16 @@ const JoinDialog = new Lang.Class({
         let isMain = page == DialogPage.MAIN;
 
         if (isMain)
-            this._details.reset();
+            this._nameEntry.grab_focus();
+        else
+            this._customToggle.active = false;
 
         this._backButton.visible = !isMain;
+        this._joinButton.visible = isMain;
         this._cancelButton.visible = isMain;
-        this.widget.title = isMain ? _("Join Chat Room")
-                                   : _("Add Connection");
-        this._confirmButton.label = isMain ? _("_Join")
-                                           : _("_Save");
-        this._stack.visible_child_name = isMain ? 'main' : 'connection';
-        this._updateCanConfirm();
+        this.title = isMain ? _("Join Chat Room")
+                            : _("Add Connection");
+        this._mainStack.visible_child_name = isMain ? 'main' : 'connection';
+        this._updateCanJoin();
     }
 });
