@@ -87,11 +87,6 @@ static GRegex *color_code_regex = NULL;
 
 G_DEFINE_TYPE_WITH_PRIVATE (PolariRoom, polari_room, G_TYPE_OBJECT)
 
-#define tp_properties_changed_cb \
-        tp_cli_dbus_properties_signal_callback_properties_changed
-#define tp_properties_get_all_cb \
-        tp_cli_dbus_properties_callback_for_get_all
-
 static void polari_room_set_channel (PolariRoom *room, TpChannel *channel);
 
 
@@ -123,6 +118,19 @@ polari_create_room_id (TpAccount    *account,
   return id;
 }
 
+#ifdef HAVE_STRCASESTR
+#  define MATCHFUNC(haystick,needle) strcasestr (haystick, needle)
+#  define MESSAGE_TO_TEXT(message) tp_message_to_text (message, NULL)
+#else
+   static inline char *
+   message_to_casefolded_text (TpMessage *message) {
+     g_autofree char *tmp = tp_message_to_text (message, NULL);
+     return g_utf8_casefold (tmp, -1);
+   }
+#  define MATCHFUNC(haystick,needle) strstr (haystick, needle)
+#  define MESSAGE_TO_TEXT(message) message_to_casefolded_text (message)
+#endif
+
 gboolean
 polari_room_should_highlight_message (PolariRoom *room,
                                       TpMessage *message)
@@ -149,9 +157,9 @@ polari_room_should_highlight_message (PolariRoom *room,
   if (tp_signalled_message_get_sender (message) == self)
     return FALSE;
 
-  text = tp_message_to_text (message, NULL);
+  text = MESSAGE_TO_TEXT (message);
   len = strlen (priv->self_nick);
-  match = strstr (text, priv->self_nick);
+  match = MATCHFUNC (text, priv->self_nick);
 
   while (match != NULL)
     {
@@ -164,7 +172,7 @@ polari_room_should_highlight_message (PolariRoom *room,
       result = starts_word && ends_word;
       if (result)
         break;
-      match = strstr (match + len, priv->self_nick);
+      match = MATCHFUNC (match + len, priv->self_nick);
     }
 
   g_free (text);
@@ -370,24 +378,33 @@ update_subject (PolariRoom *room,
                 GHashTable *properties)
 {
   PolariRoomPrivate *priv = room->priv;
-  const char *subject;
+  const char *raw_subject;
+  char *subject = NULL;
 
-  subject = tp_asv_get_string (properties, "Subject");
-  if (subject == NULL || g_strcmp0 (priv->topic, subject) == 0)
+  raw_subject = tp_asv_get_string (properties, "Subject");
+
+  if (raw_subject == NULL)
     return;
 
+  subject = strip_color_codes (raw_subject);
+  if (g_strcmp0 (priv->topic, subject) == 0)
+    {
+      g_free (subject);
+      return;
+    }
+
   g_free (priv->topic);
-  priv->topic = *subject ? g_strdup (subject) : NULL;
+  priv->topic = subject;
 
   g_object_notify_by_pspec (G_OBJECT (room), props[PROP_TOPIC]);
 }
 
 static void
-subject_get_all (TpProxy *proxy,
-                 GHashTable *properties,
-                 GError     *error,
-                 gpointer    user_data,
-                 GObject    *object)
+subject_get_all (TpProxy      *proxy,
+                 GHashTable   *properties,
+                 const GError *error,
+                 gpointer      user_data,
+                 GObject      *object)
 {
   if (error)
     return;
@@ -396,11 +413,11 @@ subject_get_all (TpProxy *proxy,
 }
 
 static void
-properties_changed (TpProxy *proxy,
-                    const char *iface_name,
-                    GHashTable *changed,
-                    const char *invalidated,
-                    gpointer    data,
+properties_changed (TpProxy     *proxy,
+                    const char  *iface_name,
+                    GHashTable  *changed,
+                    const char **invalidated,
+                    gpointer     data,
                     GObject    *weak_ref)
 {
   if (strcmp (iface_name, TP_IFACE_CHANNEL_INTERFACE_SUBJECT) != 0)
@@ -566,8 +583,7 @@ polari_room_set_channel (PolariRoom *room,
       else
         tp_cli_dbus_properties_call_get_all (channel, -1,
                                      TP_IFACE_CHANNEL_INTERFACE_SUBJECT,
-                                     (tp_properties_get_all_cb)subject_get_all,
-                                     room, NULL, NULL);
+                                     subject_get_all, room, NULL, NULL);
 
 
       priv->self_contact_notify_id =
@@ -582,8 +598,7 @@ polari_room_set_channel (PolariRoom *room,
                           G_CALLBACK (on_channel_invalidated), room);
       priv->properties_changed_id =
         tp_cli_dbus_properties_connect_to_properties_changed (
-                                 channel,
-                                 (tp_properties_changed_cb) properties_changed,
+                                 channel, properties_changed,
                                  room, NULL, NULL, NULL);
     }
 
