@@ -20,20 +20,23 @@ function _onPopoverVisibleChanged(popover) {
 
 const RoomRow = new Lang.Class({
     Name: 'RoomRow',
+    Extends: Gtk.ListBoxRow,
+    Template: 'resource:///org/gnome/Polari/ui/room-list-row.ui',
+    InternalChildren: ['eventBox', 'icon', 'roomLabel', 'counter'],
 
     _init: function(room) {
-        this._createWidget(room.icon);
-
-        let app = Gio.Application.get_default();
-        this.widget.room = room;
-        this.widget.account = room.account;
+        this.parent();
 
+        this._room = room;
         this._popover = null;
 
+        this._icon.gicon = room.icon;
+        this._icon.visible = room.icon != null;
+
         this._eventBox.connect('button-release-event',
-                            Lang.bind(this, this._onButtonRelease));
-        this.widget.connect('key-press-event',
-                            Lang.bind(this, this._onKeyPress));
+                               Lang.bind(this, this._onButtonRelease));
+        this.connect('key-press-event',
+                     Lang.bind(this, this._onKeyPress));
 
         room.connect('notify::channel', Lang.bind(this,
             function() {
@@ -50,13 +53,25 @@ const RoomRow = new Lang.Class({
         this._updatePending();
     },
 
+    get room() {
+        return this._room;
+    },
+
+    get account() {
+        return this._room.account;
+    },
+
+    get hasPending() {
+        return !this.get_style_context().has_class('inactive');
+    },
+
     selected: function() {
-        if (!this.widget.room.channel)
+        if (!this._room.channel)
             this._updatePending();
     },
 
     _updatePending: function() {
-        let room = this.widget.room;
+        let room = this._room;
 
         let pending;
         let numPendingHighlights;
@@ -77,7 +92,7 @@ const RoomRow = new Lang.Class({
         this._counter.label = numPendingHighlights.toString();
         this._counter.opacity = numPendingHighlights > 0 ? 1. : 0.;
 
-        let context = this.widget.get_style_context();
+        let context = this.get_style_context();
         if (pending.length == 0)
             context.add_class('inactive');
         else
@@ -109,53 +124,16 @@ const RoomRow = new Lang.Class({
 
     _showPopover: function() {
         if (!this._popover) {
-            let room = this.widget.room;
             let menu = new Gio.Menu();
-            menu.append(room.type == Tp.HandleType.ROOM ? _("Leave chatroom")
-                                                        : _("End conversation"),
-                        'app.leave-room(("%s", ""))'.format(room.id));
+            let isRoom = this._room.type == Tp.HandleType.ROOM;
+            menu.append(isRoom ? _("Leave chatroom") : _("End conversation"),
+                        'app.leave-room(("%s", ""))'.format(this._room.id));
 
-            this._popover = Gtk.Popover.new_from_model(this.widget, menu);
+            this._popover = Gtk.Popover.new_from_model(this, menu);
             this._popover.connect('notify::visible', _onPopoverVisibleChanged);
             this._popover.position = Gtk.PositionType.BOTTOM;
         }
         this._popover.show();
-    },
-
-    _createWidget: function(gicon) {
-        this.widget = new Gtk.ListBoxRow({ margin_bottom: 4,
-                                           focus_on_click: false });
-
-        this._eventBox = new Gtk.EventBox();
-        this.widget.add(this._eventBox);
-
-        let box = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL,
-                                margin_start: 8, margin_end: 8,
-                                margin_top: 2, margin_bottom: 2, spacing: 6 });
-        this._eventBox.add(box);
-
-        if (gicon) {
-            let icon = new Gtk.Image({ gicon: gicon,
-                                       icon_size: Gtk.IconSize.MENU,
-                                       valign: Gtk.Align.BASELINE });
-            box.add(icon);
-        }
-
-        this._roomLabel = new Gtk.Label({ hexpand: true,
-                                          ellipsize: Pango.EllipsizeMode.END,
-                                          halign: Gtk.Align.START,
-                                          valign: Gtk.Align.BASELINE });
-        box.add(this._roomLabel);
-
-        let frame = new Gtk.AspectFrame({ obey_child: false,
-                                          shadow_type: Gtk.ShadowType.NONE });
-        box.add(frame);
-
-        this._counter = new Gtk.Label({ width_chars: 2 });
-        this._counter.get_style_context().add_class('pending-messages-count');
-        frame.add(this._counter);
-
-        this.widget.show_all();
     }
 });
 
@@ -163,11 +141,12 @@ const RoomListHeader = new Lang.Class({
     Name: 'RoomListHeader',
     Extends: Gtk.MenuButton,
     CssName: 'row',
-    Template: 'resource:///org/gnome/Polari/room-list-header.ui',
+    Template: 'resource:///org/gnome/Polari/ui/room-list-header.ui',
     InternalChildren: ['label',
                        'iconStack',
                        'popoverStatus',
                        'popoverTitle',
+                       'popoverPassword',
                        'popoverReconnect',
                        'popoverRemove',
                        'popoverProperties',
@@ -182,13 +161,28 @@ const RoomListHeader = new Lang.Class({
 
         this.parent(params);
 
+        this.popover.set_default_widget(this._popoverPassword);
         this.popover.connect('notify::visible', _onPopoverVisibleChanged);
+        this.popover.connect('closed', Lang.bind(this,
+            function() {
+                this._popoverPassword.text = '';
+            }));
 
         let target = new GLib.Variant('o', this._account.get_object_path());
         this._popoverReconnect.action_target = target;
         this._popoverRemove.action_target = target;
         this._popoverProperties.action_target = target;
 
+        this._popoverPassword.connect('activate', Lang.bind(this,
+            function() {
+                let action = this._app.lookup_action('authenticate-account');
+                let password = this._popoverPassword.text;
+                let accountPath = this._account.get_object_path();
+                let param = new GLib.Variant('(os)', [accountPath, password]);
+                action.activate(param);
+                this.popover.hide();
+            }));
+
         let displayNameChangedId =
             this._account.connect('notify::display-name',
                                   Lang.bind(this, this._onDisplayNameChanged));
@@ -236,17 +230,22 @@ const RoomListHeader = new Lang.Class({
     _onConnectionStatusChanged: function() {
         let status = this._account.connection_status;
         let reason = this._account.connection_status_reason;
+        let authError = Tp.error_get_dbus_name(Tp.Error.AUTHENTICATION_FAILED);
         let isError = (status == Tp.ConnectionStatus.DISCONNECTED &&
                        reason != Tp.ConnectionStatusReason.REQUESTED);
-        let child = 'none';
+        let isAuth = isError && this._account.connection_error == authError;
+
+        let child = 'default';
         if (status == Tp.ConnectionStatus.CONNECTING) {
             if (this._networkMonitor.network_available)
                 child = 'connecting';
         } else if (isError) {
-            child = 'error';
+            child = isAuth ? 'auth' : 'error';
         }
+
         this._iconStack.visible_child_name = child;
         this._spinner.active = (child == 'connecting');
+        this._popoverTitle.visible = !isAuth;
 
         this._popoverTitle.use_markup = isError;
         this._popoverStatus.use_markup = !isError;
@@ -305,7 +304,7 @@ const RoomListHeader = new Lang.Class({
                 return _("Could not connect to %s in a safe way.").format(this._account.display_name);
 
             case Tp.error_get_dbus_name(Tp.Error.AUTHENTICATION_FAILED):
-                return _("Could not connect to %s. Authentication failed.").format(this._account.display_name);
+                return _("%s requires a password.").format(this._account.display_name);
 
             case Tp.error_get_dbus_name(Tp.Error.CONNECTION_FAILED):
             case Tp.error_get_dbus_name(Tp.Error.CONNECTION_LOST):
@@ -321,22 +320,16 @@ const RoomListHeader = new Lang.Class({
 
 const RoomList = new Lang.Class({
     Name: 'RoomList',
+    Extends: Gtk.ListBox,
 
-    _init: function() {
-        this.widget = new Gtk.ListBox({ hexpand: false });
-        this.widget.get_style_context().add_class('sidebar');
+    _init: function(params) {
+        this.parent(params);
 
-        this.widget.set_selection_mode(Gtk.SelectionMode.BROWSE);
-        this.widget.set_header_func(Lang.bind(this, this._updateHeader));
-        this.widget.set_sort_func(Lang.bind(this, this._sort));
+        this.set_header_func(Lang.bind(this, this._updateHeader));
+        this.set_sort_func(Lang.bind(this, this._sort));
 
         this._placeholders = {};
         this._roomRows = {};
-        this._selectedRows = 0;
-        this._selectionMode = false;
-
-        this.widget.connect('row-selected',
-                            Lang.bind(this, this._onRowSelected));
 
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._accountsMonitor.connect('account-manager-prepared', Lang.bind(this,
@@ -386,17 +379,17 @@ const RoomList = new Lang.Class({
         action = app.lookup_action('first-room');
         action.connect('activate', Lang.bind(this,
             function() {
-                let row = this.widget.get_row_at_index(0);
+                let row = this._getRoomRowAtIndex(0);
                 if (row)
-                    this.widget.select_row(row);
+                    this.select_row(row);
             }));
         action = app.lookup_action('last-room');
         action.connect('activate', Lang.bind(this,
             function() {
                 let nRows = this._roomManager.roomCount;
-                let row = this.widget.get_row_at_index(nRows - 1);
+                let row = this._getRoomRowAtIndex(nRows - 1);
                 if (row)
-                    this.widget.select_row(row);
+                    this.select_row(row);
             }));
         action = app.lookup_action('nth-room');
         action.connect('activate', Lang.bind(this,
@@ -404,31 +397,71 @@ const RoomList = new Lang.Class({
                 let n = param.get_int32();
                 if (n > this._roomManager.roomCount)
                     return;
-                this.widget.select_row(this.widget.get_row_at_index(n - 1));
+                this.select_row(this._getRoomRowAtIndex(n - 1));
+            }));
+        action = app.lookup_action('next-pending-room');
+        action.connect('activate', Lang.bind(this,
+            function() {
+                this._moveSelectionFull(Gtk.DirectionType.DOWN,
+                                        (row) => { return row.hasPending; });
+            }));
+        action = app.lookup_action('previous-pending-room');
+        action.connect('activate', Lang.bind(this,
+            function() {
+                this._moveSelectionFull(Gtk.DirectionType.UP,
+                                        (row) => { return row.hasPending; });
             }));
     },
 
     _onLeaveActivated: function(action, param) {
         let [id, ] = param.deep_unpack();
-        let row = this._roomRows[id].widget;
+        let row = this._roomRows[id];
 
         this._moveSelectionFromRow(row);
         row.hide();
     },
 
+    _rowToRoomIndex: function(index) {
+        let nPlaceholders = Object.keys(this._placeholders).filter(
+            Lang.bind(this, function(a) {
+                return this._placeholders[a].get_index() < index;
+            })).length;
+        return index - nPlaceholders;
+    },
+
+    _roomToRowIndex: function(index) {
+        let nChildren = this.get_children().length;
+        for (let i = 0, roomIndex = 0; i < nChildren; i++)
+            if (this.get_row_at_index(i).room && roomIndex++ == index)
+                return i;
+        return -1;
+    },
+
+    _getRoomRowAtIndex: function(index) {
+        return this.get_row_at_index(this._roomToRowIndex(index));
+    },
+
     _moveSelection: function(direction) {
-        let current = this.widget.get_selected_row();
+        this._moveSelectionFull(direction, () => { return true; });
+    },
+
+    _moveSelectionFull: function(direction, testFunction){
+        let current = this.get_selected_row();
         if (!current)
             return;
+
         let inc = direction == Gtk.DirectionType.UP ? -1 : 1;
-        let index = current.get_index();
-        let row;
+        let index = this._rowToRoomIndex(current.get_index());
+
+        let row = current;
+
         do {
             index += inc;
-            row = this.widget.get_row_at_index(index);
-        } while (row && !row.room);
+            row = this._getRoomRowAtIndex(index);
+        } while (row && !testFunction(row));
+
         if (row)
-            this.widget.select_row(row);
+            this.select_row(row);
     },
 
     _moveSelectionFromRow: function(row) {
@@ -436,25 +469,26 @@ const RoomList = new Lang.Class({
             return;
 
         let activeRoom = this._roomManager.getActiveRoom();
-        let current = this._roomRows[activeRoom.id].widget;
+        let current = this._roomRows[activeRoom.id];
 
         if (current != row)
             return;
 
-        let selected = this.widget.get_selected_row();
+        let selected = this.get_selected_row();
         let newActive = null;
 
-        this.widget.select_row(row);
-        this._moveSelection(row.get_index() == 0 ? Gtk.DirectionType.DOWN
-                                                 : Gtk.DirectionType.UP);
+        let index = this._rowToRoomIndex(row.get_index());
+        this.select_row(row);
+        this._moveSelection(index == 0 ? Gtk.DirectionType.DOWN
+                                       : Gtk.DirectionType.UP);
 
-        let newSelected = this.widget.get_selected_row();
+        let newSelected = this.get_selected_row();
         if (newSelected != row)
             newActive = newSelected.room;
         this._roomManager.setActiveRoom(newActive);
 
         if (selected != row)
-            this.widget.select_row(selected);
+            this.select_row(selected);
     },
 
     _accountAdded: function(am, account) {
@@ -466,11 +500,11 @@ const RoomList = new Lang.Class({
         placeholder.account = account;
 
         this._placeholders[account] = placeholder;
-        this.widget.add(placeholder);
+        this.add(placeholder);
 
         placeholder.connect('notify::visible', Lang.bind(this,
             function() {
-                this.widget.invalidate_sort();
+                this.invalidate_sort();
             }));
 
         this._updatePlaceholderVisibility(account);
@@ -487,11 +521,11 @@ const RoomList = new Lang.Class({
     },
 
     _roomAdded: function(roomManager, room) {
-        let roomRow = new RoomRow(room);
-        this.widget.add(roomRow.widget);
-        this._roomRows[room.id] = roomRow;
+        let row = new RoomRow(room);
+        this.add(row);
+        this._roomRows[room.id] = row;
 
-        roomRow.widget.connect('destroy', Lang.bind(this,
+        row.connect('destroy', Lang.bind(this,
             function(w) {
                 delete this._roomRows[w.room.id];
             }));
@@ -499,12 +533,12 @@ const RoomList = new Lang.Class({
     },
 
     _roomRemoved: function(roomManager, room) {
-        let roomRow = this._roomRows[room.id];
-        if (!roomRow)
+        let row = this._roomRows[room.id];
+        if (!row)
             return;
 
-        this._moveSelectionFromRow(roomRow.widget);
-        roomRow.widget.destroy();
+        this._moveSelectionFromRow(row);
+        row.destroy();
         delete this._roomRows[room.id];
         this._updatePlaceholderVisibility(room.account);
     },
@@ -518,7 +552,7 @@ const RoomList = new Lang.Class({
         let ids = Object.keys(this._roomRows);
         let hasRooms = ids.some(Lang.bind(this,
             function(id) {
-                return this._roomRows[id].widget.account == account;
+                return this._roomRows[id].account == account;
             }));
         this._placeholders[account].visible = !hasRooms;
     },
@@ -526,20 +560,19 @@ const RoomList = new Lang.Class({
     _activeRoomChanged: function(roomManager, room) {
         if (!room)
             return;
-        let roomRow = this._roomRows[room.id];
-        if (!roomRow)
+        let row = this._roomRows[room.id];
+        if (!row)
             return;
 
-        let row = roomRow.widget;
         row.can_focus = false;
-        this.widget.select_row(row);
+        this.select_row(row);
         row.can_focus = true;
     },
 
-    _onRowSelected: function(w, row) {
+    vfunc_row_selected: function(row) {
         this._roomManager.setActiveRoom(row ? row.room : null);
         if (row)
-            this._roomRows[row.room.id].selected();
+            row.selected();
     },
 
     _updateHeader: function(row, before) {
@@ -549,12 +582,15 @@ const RoomList = new Lang.Class({
         let beforeAccount = getAccount(before);
         let account = getAccount(row);
 
+        let oldHeader = row.get_header();
+
         if (beforeAccount == account) {
-            row.set_header(null);
+            if (oldHeader)
+                oldHeader.destroy();
             return;
         }
 
-        if (row.get_header())
+        if (oldHeader)
             return;
 
         let roomListHeader = new RoomListHeader({ account: account });
@@ -571,8 +607,20 @@ const RoomList = new Lang.Class({
         if (hasRooms1 != hasRooms2)
             return hasRooms1 ? -1 : 1;
 
-        if (account1 != account2)
-            return account1.display_name.localeCompare(account2.display_name);
+
+        if (account1 != account2) {
+            let displayName1 = account1.display_name;
+            let displayName2 = account2.display_name;
+
+            if (displayName1 != displayName2)
+                return displayName1.localeCompare(displayName2);
+
+            // Different account with the same display name :-(
+            // Fall back to the object path to guarantee a stable sort order
+            let accountPath1 = account1.get_path_suffix();
+            let accountPath2 = account2.get_path_suffix();
+            return accountPath1.localeCompare(accountPath2);
+        }
 
         let room1 = row1.room;
         let room2 = row2.room;
