@@ -25,12 +25,25 @@ const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const Soup = imports.gi.Soup;
+const Secret = imports.gi.Secret;
 const Tp = imports.gi.TelepathyGLib;
 
 const AppNotifications = imports.appNotifications;
 const Signals = imports.signals;
 
-const GPASTE_BASEURL = 'https://paste.gnome.org/'
+const SECRET_SCHEMA = new Secret.Schema(
+    'org.gnome.Polari.Account', Secret.SchemaFlags.NONE,
+    { 'account-id': Secret.SchemaAttributeType.STRING }
+);
+
+const GPASTE_BASEURL = 'https://paste.gnome.org/';
+
+// Silly paste.gnome.org limitation:
+// http://sayakb.github.io/sticky-notes/pages/api/#create-return-values-on-error
+// The visible title is even more limited than the 30-character hard limit ...
+const MAX_PASTE_TITLE_LENGTH = 25;
+
+const IMGUR_CLIENT_ID = '4109e59177ec95e';
 
 // http://daringfireball.net/2010/07/improved_regex_for_matching_urls
 const _balancedParens = '\\((?:[^\\s()<>]+|(?:\\(?:[^\\s()<>]+\\)))*\\)';
@@ -109,6 +122,38 @@ function getTpEventTime() {
     return Tp.user_action_time_from_x11 (time);
 }
 
+function storeAccountPassword(account, password, callback) {
+    let attr = { 'account-id': account.get_path_suffix() };
+    let label = _("Polari server password for %s").format(account.display_name);
+    Secret.password_store(SECRET_SCHEMA, attr, Secret.COLLECTION_DEFAULT,
+                          label, password, null,
+        function(o, res) {
+            try {
+                let success = Secret.password_store_finish(res);
+                callback(success);
+            } catch(e) {
+                log('Failed to store password for account "%s": %s'.format(
+                    account.display_name, e.message));
+                callback(false);
+            }
+        });
+}
+
+function lookupAccountPassword(account, callback) {
+    let attr = { 'account-id': account.get_path_suffix() };
+    Secret.password_lookup(SECRET_SCHEMA, attr, null,
+        function(o, res) {
+            try {
+                let password = Secret.password_lookup_finish(res);
+                callback(password);
+            } catch(e) {
+                log('Failed to lookup password for account "%s": %s'.format(
+                    account.display_name, e.message));
+                callback(null);
+            }
+        });
+}
+
 // findUrls:
 // @str: string to find URLs in
 //
@@ -138,6 +183,9 @@ function openURL(url, timestamp) {
 }
 
 function gpaste(text, title, callback) {
+    if (title.length > MAX_PASTE_TITLE_LENGTH)
+        title = title.substr(0, MAX_PASTE_TITLE_LENGTH - 1) + '…';
+
     let params = {
         title: title,
         data: text,
@@ -166,3 +214,41 @@ function gpaste(text, title, callback) {
                 callback(null);
         });
 }
+
+function imgurPaste(pixbuf, title, callback) {
+    let [success, buffer] = pixbuf.save_to_bufferv('png', [], []);
+    if (!success) {
+        callback(null);
+        return;
+    }
+
+    let params = {
+        title: title,
+        image: GLib.base64_encode(buffer)
+    };
+
+    let session = new Soup.Session();
+    let createUrl = 'https://api.imgur.com/3/image';
+    let message = Soup.form_request_new_from_hash('POST', createUrl, params);
+
+    let requestHeaders = message.request_headers;
+    requestHeaders.append('Authorization', 'Client-ID ' + IMGUR_CLIENT_ID);
+    session.queue_message(message,
+        function(session, message) {
+            if (message.status_code != Soup.KnownStatusCode.OK) {
+                callback(null);
+                return;
+            }
+
+            let info = {};
+            try {
+                info = JSON.parse(message.response_body.data);
+            } catch(e) {
+                log(e.message);
+            }
+            if (info.success)
+                callback(info.data.link);
+            else
+                callback(null);
+        });
+}
